Package: IDSpatialStats
Version: 0.4.0
Date: 2024-07-22
Title: Estimate Global Clustering in Infectious Disease
Authors@R: c(
  person(given="John", family="Giles", role=c("aut", "ctb"), email="jrgiles@uw.edu"),
  person(given="Henrik", family="Salje", role=c("aut", "ctb"), email="hs743@cam.ac.uk"),
  person(given="Justin", family="Lessler", role=c("aut", "cre"), email="jlessler@unc.edu"),
  person(given="Timothy", family="Willard", role="ctb", email="twillard@unc.edu"))
License: GPL (>= 2)
Description: Implements various novel and standard clustering statistics and
  other analyses useful for understanding the spread of infectious disease.
RoxygenNote: 7.3.1
Encoding: UTF-8
LazyData: true
Imports: igraph, spatstat.explore, spatstat.geom
Depends: doParallel, foreach, parallel, R (>= 2.10)
Suggests: knitr, testthat (>= 3.0.0)
Config/testthat/edition: 3
NeedsCompilation: yes
Packaged: 2024-07-24 14:42:36 UTC; twillard
Author: John Giles [aut, ctb],
  Henrik Salje [aut, ctb],
  Justin Lessler [aut, cre],
  Timothy Willard [ctb]
Maintainer: Justin Lessler <jlessler@unc.edu>
Repository: CRAN
Date/Publication: 2024-07-28 20:50:07 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-04-26 13:32:13 UTC; unix
Archs: IDSpatialStats.so.dSYM
